@@ -1,36 +1,26 @@
 #!/bin/bash
 set -e
-#### $Id:build_postgis210.sh 10208 2012-08-30 05:38:22Z robe $
-#all these get passed in by jenkins
-#export OS_BUILD=64
-#export PG_VER=9.2beta2
-#export PGHOST=localhost
-#export PGPORT=8442
-#export PGUSER=postgres
-#POSTGIS_SVN_REVISION=passed_in_by_buildbot
-#POSTGIS_MAJOR_VERSION=2
-#POSTGIS_MINOR_VERSION=1
-#POSTGIS_MICRO_VERSION=0SVN
-#export GCC_TYPE=gcc48  #for pre-4.8.0 compiles this is blank
 if  [[ "${OVERRIDE}" == '' ]] ; then
-export GEOS_VER=3.8.2
-export GDAL_VER=3.4.2
-export PROJ_VER=7.2.1
-export SFCGAL_VER=1.4.1
-export PCRE_VER=8.33
-export PROTOBUF_VER=3.2.0
-export PROTOBUFC_VER=1.2.1
-export CGAL_VER=5.3
-export BOOST_VER=1.78.0
-	#BOOST_VER_WU=1_49_0
-export BOOST_VER_WU=1_78_0
+	export GEOS_VER=3.8.0
+	export GDAL_VER=2.4.3
+	export PROJ_VER=5.2.0
+	export SFCGAL_VER=1.3.2
+	export CGAL_VER=4.11
+	export ICON_VER=1.15
+	export ZLIB_VER=1.2.11
+  export PROTOBUF_VER=3.2.0
+	export PROTOBUFC_VER=1.2.1
+	export JSON_VER=0.12
+	export PROJSO=libproj-13.dll
 fi;
+
 export PROTOBUF_VER=3.2.0
 export PROTOBUFC_VER=1.2.1
 export JSON_VER=0.12
 export PCRE_VER=8.33
+
 if  [[ "${ICON_VER}" == '' ]] ; then
-  export ICON_VER=1.16
+  export ICON_VER=1.15
 fi;
 
 echo "ICON_VER ${ICON_VER}"
@@ -40,57 +30,51 @@ if  [[ "${ZLIB_VER}" == '' ]] ; then
   export ZLIB_VER=1.2.11
 fi;
 
-if  [[ "${BOOST_VER}" == '' ]] ; then
-  export BOOST_VER=1.78.0
-  export BOOST_VER_WU=1_78_0
-fi;
-
 
 #set to something even if override is on but not set
 if  [[ "${LIBXML_VER}" == '' ]] ; then
-  export LIBXML_VER=2.9.9
+  export LIBXML_VER=2.7.8
 fi;
 
 #set to something even if override is on but not set
 if  [[ "${CGAL_VER}" == '' ]] ; then
-  export CGAL_VER=5.3
+  export CGAL_VER=4.11
 fi;
 
+#export GDAL_VER=2.4.0
+#export PROJ_VER=5.2.0
 
-
-
-
-if [[ "${GCC_TYPE}" == *gcc48* ]] ; then
-	export PROJECTS=/projects
-	export MINGPROJECTS=/projects
-else
-	export PROJECTS=/projects
-	export MINGPROJECTS=/projects
-	#export JSON_VER=0.9
-fi;
+echo "ZLIB_VER $ZLIB_VER"
+echo "PROJ_VER $PROJ_VER"
+echo "LIBXML_VER $LIBXML_VER"
+echo "CGAL_VER $CGAL_VER"
+export PROJECTS=/projects
+export MINGPROJECTS=/projects
 export PATHOLD=$PATH
 
-
 if [ "$OS_BUILD" == "64" ] ; then
 	export MINGHOST=x86_64-w64-mingw32
 else
 	export MINGHOST=i686-w64-mingw32
 fi;
 
-#export PATHOLD="/mingw/bin:/mingw/include:/c/Windows/system32:/c/Windows:.:/bin:/include:/usr/local/bin:/c/ming${OS_BUILD}/svn"
-
+echo PATH AFTER: $PATH
+#export PG_VER=9.2beta2
 export PGWINVER=${PG_VER}edb
+export WORKSPACE=`pwd`
 
 echo PATH BEFORE: $PATH
 
-
+#export PGHOST=localhost
+#export PGPORT=8442
+export PGUSER=postgres
+#export GEOS_VER=3.4.0dev
+#export GDAL_VER=1.9.1
 export PGPATH=${PROJECTS}/postgresql/rel/pg${PG_VER}w${OS_BUILD}${GCC_TYPE}
-#export PROJSO=libproj-0.dll
+export PGPATHEDB=${PROJECTS}/postgresql/rel/pg${PG_VER}w${OS_BUILD}${GCC_TYPE}edb
 
 export POSTGIS_VER=${POSTGIS_MAJOR_VERSION}.${POSTGIS_MINOR_VERSION}
-
 export POSTGIS_MICRO_VER=${POSTGIS_MAJOR_VERSION}.${POSTGIS_MINOR_VERSION}.${POSTGIS_MICRO_VERSION}
-echo POSTGIS_MICRO_VERSION: $POSTGIS_VER
 
 if [ -n "$SOURCE_FOLDER" ]; then
   export POSTGIS_SRC=${PROJECTS}/postgis/$SOURCE_FOLDER
@@ -105,18 +89,12 @@ fi;
 
 #export POSTGIS_SRC=${PROJECTS}/postgis/trunk
 export GDAL_DATA="${PROJECTS}/gdal/rel-${GDAL_VER}w${OS_BUILD}${GCC_TYPE}/share/gdal"
-# export REL_PGVER=$(echo $PG_VER | tr '.' '')
-# echo $REL_PGVER
-# export RELDIR=${MINGPROJECTS}/postgis/builds/postgis-21
-# export RELVERDIR=postgis-pg${REL_PGVER}-binaries-${POSTGIS_MICRO_VER}w64
 
-#export PATH="${PATHOLD}:${PGPATH}/bin:${PGPATH}/lib"
+export RELVERDIR=postgis-pg${REL_PGVER}-binaries-${POSTGIS_MICRO_VER}w${OS_BUILD}
+
 export PATH="${PATHOLD}:${PGPATH}/bin:${PGPATH}/lib"
-
 export PATH="${PROJECTS}/xsltproc:${PROJECTS}/gtkw${OS_BUILD}${GCC_TYPE}/bin:${PROJECTS}/geos/rel-${GEOS_VER}w${OS_BUILD}${GCC_TYPE}/bin:${PROJECTS}/gdal/rel-${GDAL_VER}w${OS_BUILD}${GCC_TYPE}/bin:${PROJECTS}/rel-libiconv-${ICON_VER}w${OS_BUILD}${GCC_TYPE}/lib:${PROJECTS}/rel-libiconv-${ICON_VER}w${OS_BUILD}${GCC_TYPE}/include:${PROJECTS}/rel-libiconv-${ICON_VER}w${OS_BUILD}${GCC_TYPE}/bin:${MINGPROJECTS}/proj/rel-${PROJ_VER}w${OS_BUILD}${GCC_TYPE}/bin:${PROJECTS}/libxml/rel-libxml2-${LIBXML_VER}w${OS_BUILD}${GCC_TYPE}/bin:${PROJECTS}/zlib/rel-zlib-${ZLIB_VER}w${OS_BUILD}${GCC_TYPE}/bin:${PROJECTS}/zlib/rel-zlib-${ZLIB_VER}w${OS_BUILD}${GCC_TYPE}/lib:${PATH}"
-export PKG_CONFIG_PATH="${PROJECTS}/sqlite/rel-sqlite3w${OS_BUILD}${GCC_TYPE}/lib/pkgconfig:${PROJECTS}/zlib/rel-zlib-${ZLIB_VER}w${OS_BUILD}${GCC_TYPE}/lib:${PROJECTS}/protobuf/rel-${PROTOBUF_VER}w${OS_BUILD}${GCC_TYPE}/lib/pkgconfig:${PROJECTS}/json-c/rel-${JSON_VER}w${OS_BUILD}${GCC_TYPE}/lib/pkgconfig:${PROJECTS}/proj/rel-${PROJ_VER}w${OS_BUILD}${GCC_TYPE}/lib/pkgconfig:${PROJECTS}/gdal/rel-${GDAL_VER}w${OS_BUILD}${GCC_TYPE}/lib/pkgconfig:${PROJECTS}/pcre/rel-${PCRE_VER}w${OS_BUILD}${GCC_TYPE}/lib/pkgconfig:${PROJECTS}/zlib/rel-zlib-${ZLIB_VER}w${OS_BUILD}${GCC_TYPE}/lib/pkgconfig:${PROJECTS}/gtkw${OS_BUILD}${GCC_TYPE}/lib/pkgconfig:/mingw/${MINGHOST}/lib/pkgconfig"
-export PROJ_LIB=${PROJECTS}/proj/rel-${PROJ_VER}w${OS_BUILD}${GCC_TYPE}/share/proj
-
+export PKG_CONFIG_PATH="${PROJECTS}/sqlite/rel-sqlite3w${OS_BUILD}${GCC_TYPE}/lib/pkgconfig:${PROJECTS}/zlib/rel-zlib-${ZLIB_VER}w${OS_BUILD}${GCC_TYPE}/lib:${PROJECTS}/protobuf/rel-${PROTOBUF_VER}w${OS_BUILD}${GCC_TYPE}/lib/pkgconfig:${PROJECTS}/json-c/rel-${JSON_VER}w${OS_BUILD}${GCC_TYPE}/lib/pkgconfig:${PROJECTS}/proj/rel-${PROJ_VER}w${OS_BUILD}${GCC_TYPE}/lib/pkgconfig:${PROJECTS}/gdal/rel-${GDAL_VER}w${OS_BUILD}${GCC_TYPE}/lib/pkgconfig:${PROJECTS}/pcre/rel-${PCRE_VER}w${OS_BUILD}${GCC_TYPE}/lib/pkgconfig:${PROJECTS}/zlib/rel-zlib-${ZLIB_VER}w${OS_BUILD}${GCC_TYPE}/lib/pkgconfig::${PROJECTS}/gtkw${OS_BUILD}${GCC_TYPE}/lib/pkgconfig"
 export SHLIB_LINK="-static-libstdc++ -lstdc++ -Wl,-Bdynamic -lm"
 CPPFLAGS="-I${PGPATH}/include -I${PROJECTS}/rel-libiconv-${ICON_VER}w${OS_BUILD}${GCC_TYPE}/include"
 
@@ -148,13 +126,22 @@ if [ -n "$PCRE_VER" ]; then
     export PATH="${PROJECTS}/pcre/rel-${PCRE_VER}w${OS_BUILD}${GCC_TYPE}/include:${PROJECTS}/pcre/rel-${PCRE_VER}w${OS_BUILD}${GCC_TYPE}/lib:${PATH}"
 fi
 
+
+
 if [ -n "$SFCGAL_VER" ]; then
 	##hard code versions of cgal etc. for now
-	#export CGAL_VER=4.11
-	#BOOST_VER=1.53.0
+	export CGAL_VER=4.11
+	BOOST_VER=1.53.0
 	#BOOST_VER_WU=1_49_0
+	export BOOST_VER_WU=1_53_0
 	export PATH="${PROJECTS}/CGAL/rel-cgal-${CGAL_VER}w${OS_BUILD}${GCC_TYPE}/bin:${PROJECTS}/CGAL/rel-sfcgal-${SFCGAL_VER}w${OS_BUILD}${GCC_TYPE}/bin:${PROJECTS}/boost/rel-${BOOST_VER_WU}w${OS_BUILD}${GCC_TYPE}/lib:${PATH}"
 
+
+#CPPFLAGS="-I${PGPATH}/include -I${PROJECTS}/rel-libiconv-${ICON_VER}w${OS_BUILD}${GCC_TYPE}/include" \
+#CFLAGS="-Wall -fno-omit-frame-pointer"
+
+#LDFLAGS="-Wl,--enable-auto-import -L${PGPATH}/lib -L${PROJECTS}/gdal/rel-${GDAL_VER}w${OS_BUILD}${GCC_TYPE}/lib -L${PROJECTS}/rel-libiconv-${ICON_VER}w${OS_BUILD}${GCC_TYPE}/lib" \
+
 LDFLAGS="-Wl,--enable-auto-import -L${PGPATH}/lib -L${PROJECTS}/rel-libiconv-${ICON_VER}w${OS_BUILD}${GCC_TYPE}/lib -L${PROJECTS}/zlib/rel-zlib-${ZLIB_VER}w${OS_BUILD}${GCC_TYPE}/lib" \
 ./configure \
   --host=${MINGHOST} --with-xml2config=${PROJECTS}/libxml/rel-libxml2-${LIBXML_VER}w${OS_BUILD}${GCC_TYPE}/bin/xml2-config  \
@@ -183,12 +170,10 @@ LDFLAGS="-L${PGPATH}/lib -L${PROJECTS}/gdal/rel-${GDAL_VER}w${OS_BUILD}${GCC_TYP
   --prefix=${PROJECTS}/postgis/liblwgeom-${POSTGIS_VER}w${OS_BUILD}${GCC_TYPE}
 fi;
 
-
 make clean
+
 #patch liblwgeom generated make to get rid of dynamic linking
-sed -i 's/LDFLAGS += -no-undefined//g' liblwgeom/Makefile
-
-#make uninstall
-
-make && make install
+#sed -i 's/LDFLAGS += -no-undefined//g' liblwgeom/Makefile
 
+make
+make install
